import sys

sys.path.insert(1, "./simple-pokedex")

import lookup


# correct output
def test_lookup_1():
    response = lookup.pokemonLookup("ditto")
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "name": "ditto",
            "id": 132,
            "types": ["NORMAL"],
            "stats": [
                {
                    "base_stat": 48,
                    "effort": 1,
                    "stat": {"name": "hp", "url": "https://pokeapi.co/api/v2/stat/1/"},
                },
                {
                    "base_stat": 48,
                    "effort": 0,
                    "stat": {
                        "name": "attack",
                        "url": "https://pokeapi.co/api/v2/stat/2/",
                    },
                },
                {
                    "base_stat": 48,
                    "effort": 0,
                    "stat": {
                        "name": "defense",
                        "url": "https://pokeapi.co/api/v2/stat/3/",
                    },
                },
                {
                    "base_stat": 48,
                    "effort": 0,
                    "stat": {
                        "name": "special-attack",
                        "url": "https://pokeapi.co/api/v2/stat/4/",
                    },
                },
                {
                    "base_stat": 48,
                    "effort": 0,
                    "stat": {
                        "name": "special-defense",
                        "url": "https://pokeapi.co/api/v2/stat/5/",
                    },
                },
                {
                    "base_stat": 48,
                    "effort": 0,
                    "stat": {
                        "name": "speed",
                        "url": "https://pokeapi.co/api/v2/stat/6/",
                    },
                },
            ],
            "abilities": [
                {
                    "ability": {
                        "name": "limber",
                        "url": "https://pokeapi.co/api/v2/ability/7/",
                    },
                    "is_hidden": False,
                    "slot": 1,
                },
                {
                    "ability": {
                        "name": "imposter",
                        "url": "https://pokeapi.co/api/v2/ability/150/",
                    },
                    "is_hidden": True,
                    "slot": 3,
                },
            ],
        },
    }


# failed test where the search is not a real pokemon
def test_lookup_2():
    response = lookup.pokemonLookup("minecraft")
    print(response)
    assert response == {"success": False, "message": 404, "data": None}


# correct response for pikachu
def test_lookup_3():
    response = lookup.pokemonLookup("pikachu")
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "name": "pikachu",
            "id": 25,
            "types": ["ELECTRIC"],
            "stats": [
                {
                    "base_stat": 35,
                    "effort": 0,
                    "stat": {"name": "hp", "url": "https://pokeapi.co/api/v2/stat/1/"},
                },
                {
                    "base_stat": 55,
                    "effort": 0,
                    "stat": {
                        "name": "attack",
                        "url": "https://pokeapi.co/api/v2/stat/2/",
                    },
                },
                {
                    "base_stat": 40,
                    "effort": 0,
                    "stat": {
                        "name": "defense",
                        "url": "https://pokeapi.co/api/v2/stat/3/",
                    },
                },
                {
                    "base_stat": 50,
                    "effort": 0,
                    "stat": {
                        "name": "special-attack",
                        "url": "https://pokeapi.co/api/v2/stat/4/",
                    },
                },
                {
                    "base_stat": 50,
                    "effort": 0,
                    "stat": {
                        "name": "special-defense",
                        "url": "https://pokeapi.co/api/v2/stat/5/",
                    },
                },
                {
                    "base_stat": 90,
                    "effort": 2,
                    "stat": {
                        "name": "speed",
                        "url": "https://pokeapi.co/api/v2/stat/6/",
                    },
                },
            ],
            "abilities": [
                {
                    "ability": {
                        "name": "static",
                        "url": "https://pokeapi.co/api/v2/ability/9/",
                    },
                    "is_hidden": False,
                    "slot": 1,
                },
                {
                    "ability": {
                        "name": "lightning-rod",
                        "url": "https://pokeapi.co/api/v2/ability/31/",
                    },
                    "is_hidden": True,
                    "slot": 3,
                },
            ],
        },
    }



import sys

sys.path.insert(1, "./simple-pokedex")

import lookup


# catching error when wrong ability is sent
def test_lookup_1():
    response = lookup.abilityLookup("minecraft")
    assert response == {"success": False, "message": 404, "data": None}


# catching error when no ability is sent
def test_lookup_2():
    response = lookup.abilityLookup("")
    assert response == {"success": False, "message": 400, "data": None}


# correct response for moxie
def test_lookup_3():
    response = lookup.abilityLookup("moxie")
    assert response == {
        "success": True,
        "message": "success",
        "data": "This Pokémon's Attack rises one stage upon knocking out another Pokémon, even a friendly Pokémon.\n\nThis ability does not take effect when the Pokémon indirectly causes another Pokémon to faint, e.g. through poison or spikes.\n\nIf this Pokémon knocks out a Pokémon with mummy, the former's ability will change without taking effect.",
    }


# correct response for intimidate
def test_lookup_4():
    response = lookup.abilityLookup("intimidate")
    assert response == {
        "success": True,
        "message": "success",
        "data": "When this Pokémon enters battle, the opponent's Attack is lowered by one stage.  In a double battle, both opponents are affected.\n\nThis ability also takes effect when acquired during a battle, but will not take effect again if lost and reobtained without leaving battle.\n\nThis ability has no effect on an opponent that has a substitute.\n\nOverworld: If the first Pokémon in the party has this ability, any random encounter with a Pokémon five or more levels lower than it has a 50% chance of being skipped.",
    }



import requests

def poketypes(poketype):
    response = requests.get("https://pokeapi.co/api/v2/type/" + str(poketypetonum(poketype)))
    if response.status_code == 200 and poketype !=- -1:
        data = response.json()
        return {
            "success": True,
            "message": "success",
            "data": data['damage_relations']
        }
    else:
        return {"success": False, "message": response.status_code, "data": None}

def poketypetonum(poketype):
    if poketype == "normal":
        return 1
    if poketype == "fighting":
        return 2
    if poketype == "flying":
        return 3
    if poketype == "poison":
        return 4
    if poketype == "ground":
        return 5
    if poketype == "rock":
        return 6
    if poketype == "bug":
        return 7
    if poketype == "ghost":
        return 8
    if poketype == "steel":
        return 9
    if poketype == "fire":
        return 10
    if poketype == "water":
        return 11
    if poketype == "grass":
        return 12
    if poketype == "electric":
        return 13
    if poketype == "psychic":
        return 14
    if poketype == "ice":
        return 15
    if poketype == "dragon":
        return 16
    if poketype == "dark":
        return 17
    if poketype == "fairy":
        return 18
    return -1 



import requests
import random

def generate_pokemon(gen = 0, seed = 0):
    if seed != 0:
        random.seed(seed)
    if gen == 0:
        gen = random.randint(1, 9)
    r = requests.get(f"https://pokeapi.co/api/v2/generation/{gen}")
    # handle invalid input and API errors
    if r.status_code == 404:
        return {"success": False, "message": f"{r.status_code}", "data": None}
    if r.status_code != 200:
        return {"success": False, "data": f"{r.status_code}", "data": None}
    
    data = r.json()
    pokemon = random.choice(data["pokemon_species"])
    url = pokemon["url"].replace("-species", "")
    rawData = requests.get(url).json()
    pokemonData = {
        "name": rawData["name"],
        "id": rawData["id"],
        "types": [t["type"]["name"].upper() for t in rawData["types"]],
        "stats": rawData["stats"],
        "abilities": rawData["abilities"]
    }
    return {"success": True, "message": "success", "data": pokemonData}



import sys

sys.path.insert(1, "./simple-pokedex")
import poketypes


def test_poketypes():
    response = poketypes.poketypes("poison")
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "double_damage_from": [
                {"name": "ground", "url": "https://pokeapi.co/api/v2/type/5/"},
                {"name": "psychic", "url": "https://pokeapi.co/api/v2/type/14/"},
            ],
            "double_damage_to": [
                {"name": "grass", "url": "https://pokeapi.co/api/v2/type/12/"},
                {"name": "fairy", "url": "https://pokeapi.co/api/v2/type/18/"},
            ],
            "half_damage_from": [
                {"name": "fighting", "url": "https://pokeapi.co/api/v2/type/2/"},
                {"name": "poison", "url": "https://pokeapi.co/api/v2/type/4/"},
                {"name": "bug", "url": "https://pokeapi.co/api/v2/type/7/"},
                {"name": "grass", "url": "https://pokeapi.co/api/v2/type/12/"},
                {"name": "fairy", "url": "https://pokeapi.co/api/v2/type/18/"},
            ],
            "half_damage_to": [
                {"name": "poison", "url": "https://pokeapi.co/api/v2/type/4/"},
                {"name": "ground", "url": "https://pokeapi.co/api/v2/type/5/"},
                {"name": "rock", "url": "https://pokeapi.co/api/v2/type/6/"},
                {"name": "ghost", "url": "https://pokeapi.co/api/v2/type/8/"},
            ],
            "no_damage_from": [],
            "no_damage_to": [
                {"name": "steel", "url": "https://pokeapi.co/api/v2/type/9/"}
            ],
        },
    }


def test_poketypes2():
    response = poketypes.poketypes("fairy")
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "double_damage_from": [
                {"name": "poison", "url": "https://pokeapi.co/api/v2/type/4/"},
                {"name": "steel", "url": "https://pokeapi.co/api/v2/type/9/"},
            ],
            "double_damage_to": [
                {"name": "fighting", "url": "https://pokeapi.co/api/v2/type/2/"},
                {"name": "dragon", "url": "https://pokeapi.co/api/v2/type/16/"},
                {"name": "dark", "url": "https://pokeapi.co/api/v2/type/17/"},
            ],
            "half_damage_from": [
                {"name": "fighting", "url": "https://pokeapi.co/api/v2/type/2/"},
                {"name": "bug", "url": "https://pokeapi.co/api/v2/type/7/"},
                {"name": "dark", "url": "https://pokeapi.co/api/v2/type/17/"},
            ],
            "half_damage_to": [
                {"name": "poison", "url": "https://pokeapi.co/api/v2/type/4/"},
                {"name": "steel", "url": "https://pokeapi.co/api/v2/type/9/"},
                {"name": "fire", "url": "https://pokeapi.co/api/v2/type/10/"},
            ],
            "no_damage_from": [
                {"name": "dragon", "url": "https://pokeapi.co/api/v2/type/16/"}
            ],
            "no_damage_to": [],
        },
    }


def test_poketypes3():
    response = poketypes.poketypes("palworld")
    assert response == {"success": False, "message": 404, "data": None}



import sys

sys.path.insert(1, "./simple-pokedex")
import generatePokemon


def test_generate_1():
    response = generatePokemon.generate_pokemon(4, 123)
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "name": "shinx",
            "id": 403,
            "types": ["ELECTRIC"],
            "stats": [
                {
                    "base_stat": 45,
                    "effort": 0,
                    "stat": {"name": "hp", "url": "https://pokeapi.co/api/v2/stat/1/"},
                },
                {
                    "base_stat": 65,
                    "effort": 1,
                    "stat": {
                        "name": "attack",
                        "url": "https://pokeapi.co/api/v2/stat/2/",
                    },
                },
                {
                    "base_stat": 34,
                    "effort": 0,
                    "stat": {
                        "name": "defense",
                        "url": "https://pokeapi.co/api/v2/stat/3/",
                    },
                },
                {
                    "base_stat": 40,
                    "effort": 0,
                    "stat": {
                        "name": "special-attack",
                        "url": "https://pokeapi.co/api/v2/stat/4/",
                    },
                },
                {
                    "base_stat": 34,
                    "effort": 0,
                    "stat": {
                        "name": "special-defense",
                        "url": "https://pokeapi.co/api/v2/stat/5/",
                    },
                },
                {
                    "base_stat": 45,
                    "effort": 0,
                    "stat": {
                        "name": "speed",
                        "url": "https://pokeapi.co/api/v2/stat/6/",
                    },
                },
            ],
            "abilities": [
                {
                    "ability": {
                        "name": "rivalry",
                        "url": "https://pokeapi.co/api/v2/ability/79/",
                    },
                    "is_hidden": False,
                    "slot": 1,
                },
                {
                    "ability": {
                        "name": "intimidate",
                        "url": "https://pokeapi.co/api/v2/ability/22/",
                    },
                    "is_hidden": False,
                    "slot": 2,
                },
                {
                    "ability": {
                        "name": "guts",
                        "url": "https://pokeapi.co/api/v2/ability/62/",
                    },
                    "is_hidden": True,
                    "slot": 3,
                },
            ],
        },
    }


def test_generate_2():
    response = generatePokemon.generate_pokemon(5, 111)
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "name": "archen",
            "id": 566,
            "types": ["ROCK", "FLYING"],
            "stats": [
                {
                    "base_stat": 55,
                    "effort": 0,
                    "stat": {"name": "hp", "url": "https://pokeapi.co/api/v2/stat/1/"},
                },
                {
                    "base_stat": 112,
                    "effort": 1,
                    "stat": {
                        "name": "attack",
                        "url": "https://pokeapi.co/api/v2/stat/2/",
                    },
                },
                {
                    "base_stat": 45,
                    "effort": 0,
                    "stat": {
                        "name": "defense",
                        "url": "https://pokeapi.co/api/v2/stat/3/",
                    },
                },
                {
                    "base_stat": 74,
                    "effort": 0,
                    "stat": {
                        "name": "special-attack",
                        "url": "https://pokeapi.co/api/v2/stat/4/",
                    },
                },
                {
                    "base_stat": 45,
                    "effort": 0,
                    "stat": {
                        "name": "special-defense",
                        "url": "https://pokeapi.co/api/v2/stat/5/",
                    },
                },
                {
                    "base_stat": 70,
                    "effort": 0,
                    "stat": {
                        "name": "speed",
                        "url": "https://pokeapi.co/api/v2/stat/6/",
                    },
                },
            ],
            "abilities": [
                {
                    "ability": {
                        "name": "defeatist",
                        "url": "https://pokeapi.co/api/v2/ability/129/",
                    },
                    "is_hidden": False,
                    "slot": 1,
                }
            ],
        },
    }


def test_generate_3():
    response = generatePokemon.generate_pokemon(10)
    assert response == {"success": False, "message": "404", "data": None}


def test_generate_4():
    response = generatePokemon.generate_pokemon(0, 555)
    assert response == {
        "success": True,
        "message": "success",
        "data": {
            "name": "infernape",
            "id": 392,
            "types": ["FIRE", "FIGHTING"],
            "stats": [
                {
                    "base_stat": 76,
                    "effort": 0,
                    "stat": {"name": "hp", "url": "https://pokeapi.co/api/v2/stat/1/"},
                },
                {
                    "base_stat": 104,
                    "effort": 1,
                    "stat": {
                        "name": "attack",
                        "url": "https://pokeapi.co/api/v2/stat/2/",
                    },
                },
                {
                    "base_stat": 71,
                    "effort": 0,
                    "stat": {
                        "name": "defense",
                        "url": "https://pokeapi.co/api/v2/stat/3/",
                    },
                },
                {
                    "base_stat": 104,
                    "effort": 1,
                    "stat": {
                        "name": "special-attack",
                        "url": "https://pokeapi.co/api/v2/stat/4/",
                    },
                },
                {
                    "base_stat": 71,
                    "effort": 0,
                    "stat": {
                        "name": "special-defense",
                        "url": "https://pokeapi.co/api/v2/stat/5/",
                    },
                },
                {
                    "base_stat": 108,
                    "effort": 1,
                    "stat": {
                        "name": "speed",
                        "url": "https://pokeapi.co/api/v2/stat/6/",
                    },
                },
            ],
            "abilities": [
                {
                    "ability": {
                        "name": "blaze",
                        "url": "https://pokeapi.co/api/v2/ability/66/",
                    },
                    "is_hidden": False,
                    "slot": 1,
                },
                {
                    "ability": {
                        "name": "iron-fist",
                        "url": "https://pokeapi.co/api/v2/ability/89/",
                    },
                    "is_hidden": True,
                    "slot": 3,
                },
            ],
        },
    }



import requests


def pokemonLookup(pokemon=""):
    response = requests.get("https://pokeapi.co/api/v2/pokemon/" + pokemon)
    if response.status_code == 200:
        data = response.json()
        return {
            "success": True,
            "message": "success",
            "data": {
                "name": data["name"],
                "id": data["id"],
                "types": [type["type"]["name"].upper() for type in data["types"]],
                "stats": data["stats"],
                "abilities": data["abilities"],
            },
        }
    else:
        return {"success": False, "message": response.status_code, "data": None}


def abilityLookup(ability=""):
    if ability == "":
        return {"success": False, "message": 400, "data": None}
    response = requests.get("https://pokeapi.co/api/v2/ability/" + ability)
    if response.status_code == 200:
        data = response.json()
        # parse data
        parsed_data = ""
        for item in data["effect_entries"]:
            if item["language"]["name"] == "en":
                parsed_data = item["effect"]
        return {"success": True, "message": "success", "data": parsed_data}
    else:
        return {"success": False, "message": response.status_code, "data": None}


print(abilityLookup(""))
print("--")

print(abilityLookup("minecraft"))

print("--")

print(abilityLookup("moxie"))

print("--")
print(abilityLookup("intimidate"))
